@@ -36,6 +36,8 @@
 /*                                                                       */
 /* Phrase break prediction                                               */
 /*                                                                       */
+/* Modified Jul 2011: aup added support for CART phrasing (phrasyn)      */
+/*                                                                       */
 /*=======================================================================*/
 #include <cstdio>
 #include "festival.h"
@@ -45,6 +47,9 @@ static void phrasing_none(EST_Utterance *u);
 static void phrasing_by_cart(EST_Utterance *u);
 static void phrasing_by_probmodels(EST_Utterance *u);
 static void phrasing_by_fa(EST_Utterance *u);
+static void phrasing_by_cart_viterbi(EST_Utterance *u); // Addition by AUP
+static void phrasing_by_cart_probmodels_combined(EST_Utterance *u); // Addition by AUP
+static EST_VTCandidate *cart_bb_candlist(EST_Item *s, EST_Features &f); // Addition by AUP
 static EST_VTCandidate *bb_candlist(EST_Item *s,EST_Features &f);
 static EST_VTPath *bb_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f);
 static double find_b_prob(EST_VTPath *p,int n,int *state);
@@ -65,6 +70,8 @@ static LISP bb_tags = NIL;
 static LISP pos_map = NIL;
 static LISP phrase_type_tree = NIL;
 
+static LISP bb_unigrams = NIL;
+
 /* Interslice */
 static EST_Track *bb_track = 0;
 
@@ -75,13 +82,15 @@ LISP FT_Classic_Phrasify_Utt(LISP utt)
     LISP phrase_method = ft_get_param("Phrase_Method");
 
     *cdebug << "Phrasify module\n";
-
+    *cdebug << "Using method: " << get_c_string(phrase_method) << endl;
     if (u->relation_present("Phrase"))
 	return utt;               // already specified
     else if (phrase_method == NIL)
 	phrasing_none(u);  // all one phrase
     else if (streq("prob_models",get_c_string(phrase_method)))
 	phrasing_by_probmodels(u);
+    else if (streq("prob_cart_combined", get_c_string(phrase_method)))
+        phrasing_by_cart_probmodels_combined(u);
     else if (streq("cart_tree",get_c_string(phrase_method)))
 	phrasing_by_cart(u);
     else if (streq("forced_align",get_c_string(phrase_method)))
@@ -164,6 +173,9 @@ static void pbyp_get_params(LISP params)
 
     bb_name = get_param_str("break_ngram_name",params,"");
     bb_break_filename = get_param_str("break_ngram_filename",params,"");
+
+    *cdebug << "File: "<<bb_break_filename<<endl;
+
     if ((bb_ngram = get_ngram(bb_name,bb_break_filename)) == 0)
     {
 	cerr << "PHRASIFY: no ngram called \"" <<
@@ -173,6 +185,9 @@ static void pbyp_get_params(LISP params)
     bb_tags = get_param_lisp("break_tags",params,NIL);
     phrase_type_tree = get_param_lisp("phrase_type_tree",params,NIL);
 
+    *cdebug << "Tree: "<<phrase_type_tree << endl;
+    bb_unigrams = get_param_lisp("break_unigrams", params, NIL);
+
     bb_track_name = get_param_str("break_track_name",params,"");
     if (bb_track_name != "")
     {
@@ -197,6 +212,7 @@ static void pbyp_get_params(LISP params)
     l1 = siod_get_lval("pos_n_start_tag",NULL);
     if (l1 != NIL) 
 	pos_n_start_tag = bb_pos_ngram->get_vocab_word(get_c_string(l1));
+
 }
 
 static void phrasing_by_probmodels(EST_Utterance *u)
@@ -230,8 +246,8 @@ static void phrasing_by_probmodels(EST_Utterance *u)
     u->create_relation("Phrase");
     for (w=u->relation("Word")->first(); w != 0; w = w->next())
     {
-	w->set("pbreak",bb_ngram->
-		 get_vocab_word(w->f("pbreak_index").Int()));
+      w->set("pbreak",bb_ngram->
+      	 get_vocab_word(w->f("pbreak_index").Int()));
 	if (phr == 0)
 	    phr = add_phrase(u);
 	append_daughter(phr,"Phrase",w);
@@ -256,6 +272,139 @@ static void phrasing_by_probmodels(EST_Utterance *u)
     bb_tags = NIL;
 }
 
+static void phrasing_by_cart_viterbi(EST_Utterance *u)
+{
+  EST_Item *w, *phr = 0;
+  LISP tree;
+  LISP answer;
+  int num_states;
+  EST_String pbreak;
+
+  *cdebug << "Inside phrasing_by_cart_viterbi"<<endl;
+
+    pbyp_get_params(siod_get_lval("phr_break_params",NULL));
+    gc_protect(&bb_tags);
+
+    B_word = bb_ngram->get_vocab_word("B");
+    NB_word = bb_ngram->get_vocab_word("NB");
+    BB_word = bb_ngram->get_vocab_word("BB");
+
+
+    *cdebug << "States: "<<bb_ngram->num_states()<<endl;
+    *cdebug << "Order: "<<bb_ngram->order()<<endl;
+    *cdebug << "B word: "<<B_word<<endl;
+    *cdebug << "BB word: "<<BB_word<<endl;
+    *cdebug << "NB word: "<<NB_word<<endl;
+
+    num_states = bb_ngram->num_states();
+    EST_Viterbi_Decoder v(cart_bb_candlist,bb_npath,num_states);
+
+    v.initialise(u->relation("Word"));
+    v.search();
+    int x = v.result("pbreak_index");
+    *cdebug << "Viterbi result ended in "<<x<<endl;
+
+    // Given predicted break, go through and add phrases 
+    u->create_relation("Phrase");
+    for (w=u->relation("Word")->first(); w != 0; w = w->next())
+    {
+      w->set("pbreak",bb_ngram->
+	     get_vocab_word(w->f("pbreak_index").Int()));
+	if (phr == 0)
+	    phr = add_phrase(u);
+	append_daughter(phr,"Phrase",w);
+	if (phrase_type_tree != NIL)
+	{
+	    EST_Val npbreak = wagon_predict(w,phrase_type_tree);
+	    w->set("pbreak",npbreak.string());  // may reset to BB
+	    *cdebug << "Reset: " << npbreak << endl; ;
+	}
+	pbreak = (EST_String)w->f("pbreak");
+	if (pbreak == "B")
+	    w->set("blevel",3);
+	else if (pbreak == "mB")
+	    w->set("blevel",2);
+	if ((pbreak == "B") || (pbreak == "BB") || (pbreak == "mB"))
+	{
+	    phr->set_name((EST_String)pbreak);
+	    phr = 0;
+	}
+    }
+
+    gc_unprotect(&bb_tags);
+    bb_tags = NIL;
+}
+
+static EST_VTCandidate *cart_bb_candlist(EST_Item *s, EST_Features &f)
+{
+  double prob=1.0;
+  double divisor = 1.0;
+  EST_VTCandidate *all_c = 0;
+
+  EST_VTCandidate *c;
+  LISP l;
+  LISP m;
+  LISP answer, tree;
+  tree = siod_get_lval("phrase_cart_tree", "no phrase cart tree");
+
+  answer = wagon_pd(s, car(tree));
+  //*cdebug << get_c_string(answer) <<endl;
+  //*cdebug << endl;
+  
+  if (s->next() == 0)  // end of utterances so force a break
+    {   
+      EST_VTCandidate *c = new EST_VTCandidate;
+      c->s = s;
+      c->name = B_word;
+      c->score = log(0.95);  // very very likely, but not absolute
+      c->next = all_c;
+      all_c = c;
+      return all_c;
+    }
+
+
+  m = bb_unigrams;
+
+  for (l=bb_tags; l !=0; l=cdr(l))
+    {
+      if(m == NIL)
+	divisor = 1.0;
+      else 
+	{
+	  divisor = get_c_float(car(m));
+	  m = cdr(m);
+	}
+
+      c = new EST_VTCandidate;
+      c->s = s;
+      c->name = bb_ngram->get_vocab_word(get_c_string(car(l)));
+      prob = get_param_float(get_c_string(car(l)), answer, 0.001);
+
+      if (prob == 0) prob = 0.0000001;
+      if (prob == 1) prob = 0.9999999;
+      // Divide by unigram probability to get "Reverse" score      
+
+      //*cdebug << "Prob: "<<log(prob)<<endl;
+      c->score = log(prob) - log(divisor);
+      s->set("phrase_score", c->score);
+      c->next = all_c;
+      all_c = c;
+    }
+
+  return all_c;
+}
+
+
+static void phrasing_by_cart_probmodels_combined(EST_Utterance *u)
+{
+  *cdebug << "Using Phrasing Method: prob_cart_combined"<<endl;
+  
+  // Simply return the cart viterbi!
+  phrasing_by_cart_viterbi(u);
+
+}
+
+
 static EST_VTCandidate *bb_candlist(EST_Item *s,EST_Features &f)
 {
     // Find candidates with a priori probabilities
@@ -274,6 +423,7 @@ static EST_VTCandidate *bb_candlist(EST_Item *s,EST_Features &f)
 	    window[2] = s->next()->I("pos_index",0);
 	else
 	    window[2] = pos_n_start_tag;
+	*cdebug << window[0] << " " << window[1] << " " << window[2] << " " ;
     }
     else if (bb_pos_ngram->order() == 3)
     {
@@ -363,6 +513,7 @@ static EST_VTCandidate *bb_candlist(EST_Item *s,EST_Features &f)
 	    window[bb_pos_ngram->order()-1] = tag;
 	    const EST_DiscreteProbDistribution &pd = 
 		bb_pos_ngram->prob_dist(window);
+
 	    if (pd.samples() == 0)
 	    {
 		if (tag == B_word)
@@ -392,7 +543,6 @@ static EST_VTCandidate *bb_candlist(EST_Item *s,EST_Features &f)
 	    c->name = tag;
 	    window[bb_pos_ngram->order()-1] = tag;
 	    prob = bb_pos_ngram->reverse_probability(window);
-
 	    // If this word came from inside a token reduce the
 	    // probability of a break
  	    if ((ffeature(s,"R:Token.n.name") != "0") &&
@@ -440,6 +590,8 @@ static EST_VTPath *bb_npath(EST_VTPath *p,EST_VTCandidate *c,EST_Features &f)
     lang_prob = (1.0 * c->score) + gscale_p;
     lang_prob = c->score;
 
+    //*cdebug << "Addition: "<<(lang_prob+lprob)<<endl;
+
 //    np->set_feature(lscorename,lang_prob+lprob);
     if (p==0)
 	np->score = (lang_prob+lprob);
@@ -454,6 +606,18 @@ static double find_b_prob(EST_VTPath *p,int n,int *state)
     int oldstate=0;
     double prob;
 
+    
+    /*
+    //1-gram LM enforced
+     *state = 0;
+     if(n == B_word) return 0.06;
+     else return 0.94;
+    */
+    /*
+    //0-gram LM enforced
+     *state = 0;
+     return 0.5;
+    */
     if (p == 0)
     {
 	int order = bb_ngram->order();
@@ -474,9 +638,12 @@ static double find_b_prob(EST_VTPath *p,int n,int *state)
 	prob = (double)pd.probability(n);
     // This is too specific
     if (n == B_word)
-	prob *= gscale_s;
-    *state = bb_ngram->find_next_state_id(oldstate,n);
+      prob *= gscale_s;
 
+    *state = bb_ngram->find_next_state_id(oldstate,n);
+    //*cdebug << "oldstate: "<<oldstate<<" newstate: "<<*state<<" for word "<<n<<endl;
+
+    //return 0;
     return prob;
 
 }
@@ -489,7 +656,7 @@ static double find_b_faprob(EST_VTPath *p,int n,int *state)
     int i,j;
     EST_VTPath *d;
     double prob;
-    double atime, wtime, wstddev=0, z;
+    double atime, wtime, wstddev=0, z=0.0;
     static int ATOTH_BREAK=2;
     static int ATOTH_NBREAK=1;
 
